/*
 * File         : MemberSignatureImage.java
 * Author       : Guansong Zhang
 * Created      : Tue May 19 13:07:52 1998
 * Revision     : $Revision: 1.6 $
 * Updated      : $Date: 1999/03/16 20:27:41 $
 */

package hpjava.compiler.reflection;

import java.util.*;
import java.lang.reflect.*;

import hpjava.compiler.util.*;
import hpjava.compiler.syntaxtree.*;
import hpjava.compiler.visitor.*;

public class MemberSignatureImage {
  String name;
  
  MemberSignatureImage(String pname, Vector inames, ConstructorDeclaration n) {
    MemberSignatureResolver resolver = new MemberSignatureResolver(pname, inames, this);
    n.accept(resolver);  
  }

  MemberSignatureImage(String pname, Vector inames, MethodDeclaration n) {
    MemberSignatureResolver resolver = new MemberSignatureResolver(pname, inames, this);
    n.accept(resolver);  
  }
  
  public String toString() {
    return name;
  }
}

/*
  MemberSignatureImage is used to record member name in a ClassReflector.
  Its resolver creates the actural String signature.
   
*/  

/*
 * File         : MemberSignatureResolver.java
 * Author       : Guansong Zhang
 * Created      : Tue May 19 13:07:52 1998
 * Revision     : $Revision: 1.6 $
 * Updated      : $Date: 1999/03/16 20:27:41 $
 */

class MemberSignatureResolver extends DepthFirstVisitor {
  
  MemberSignatureImage msd;

  private String pname;
  private Vector inames;

  private StringBuffer nameBuffer = new StringBuffer();  

  private IntStack status = new IntStack();
  
  final int DEFAULT  = 0;
  final int NEEDNAME = 1;
  final int FORFIELD = 3;
      
  MemberSignatureResolver(String pname, Vector inames, MemberSignatureImage msd) {
    this.msd = msd;
    this.pname = pname;
    this.inames = inames;
        
    status.push(DEFAULT);
  }

   //
   // f0 -> [ "public" | "protected" | "private" ]
   // f1 -> <IDENTIFIER>
   // f2 -> FormalParameters()
   // f3 -> [ "throws" NameList() ]
   // f4 -> "{"
   // f5 -> [ ExplicitConstructorInvocation() ]
   // f6 -> ( BlockStatement() )*
   // f7 -> "}"
   //
   public void visit(ConstructorDeclaration n) {
      nameBuffer.append("<init>");
      n.f2.accept(this);
      
      msd.name=nameBuffer.toString();   
   }
   
  /*
   //
   // f0 -> ( "public" | "protected" | "private" | "static" | "final" | "transient" | "volatile" )*
   // f1 -> Type()
   // f2 -> VariableDeclarator()
   // f3 -> ( "," VariableDeclarator() )*
   // f4 -> ";"
   //
   public void visit(FieldDeclaration n) {
     n.f2.accept(this);
     
     msd.name=nameBuffer.toString();
   }
   */

   //
   // f0 -> VariableDeclaratorId()
   // f1 -> [ "=" VariableInitializer() ]
   //
   public void visit(VariableDeclarator n) {
      n.f0.accept(this);
   }

   //
   // f0 -> <IDENTIFIER>
   // f1 -> ( ArraySignature() )*
   //
   public void visit(VariableDeclaratorId n) {
     nameBuffer.append(n.f0.tokenImage);
   }

   //
   // f0 -> ( "public" | "protected" | "private" | "static" | "abstract" | "final" | "native" | "synchronized" )*
   // f1 -> ResultType()
   // f2 -> MethodDeclarator()
   // f3 -> [ "throws" NameList() ]
   // f4 -> ( Block() | ";" )
   //
   public void visit(MethodDeclaration n) {
     n.f2.accept(this); 

     msd.name=nameBuffer.toString(); 
   }
   
   //
   // f0 -> <IDENTIFIER>
   // f1 -> FormalParameters()
   // f2 -> ( ArraySignature() )*
   //
   public void visit(MethodDeclarator n) {
      nameBuffer.append(n.f0.tokenImage);      
      n.f1.accept(this);
   }
   
   //
   // f0 -> "("
   // f1 -> [ FormalParameter() ( "," FormalParameter() )* ]
   // f2 -> ")"
   //
   public void visit(FormalParameters n) {
      n.f1.accept(this);
   }

   //
   // f0 -> [ "final" ]
   // f1 -> Type()
   // f2 -> VariableDeclaratorId()
   //
   public void visit(FormalParameter n) {
      ArraySignaturesImage asd1= new ArraySignaturesImage((NodeListOptional)n.f2.f1);
      ArraySignaturesImage asd2= new ArraySignaturesImage((NodeListOptional)n.f1.f1);
      
      ArraySignaturesImage asd = new ArraySignaturesImage(asd1, asd2);
      
      ScalarReflectorDescriptor srd = null;
      
      switch (n.f1.f0.which) { 
      case 0:
        srd = new PrimitiveTypeDescriptor((PrimitiveType)n.f1.f0.choice);
        break;
      case 1:
        srd = new ClassTypeDescriptor(pname, inames, (Name)n.f1.f0.choice); 
        break;
      default:
        throw new Error();
      }
      
      if (asd.isEmpty()) {
        nameBuffer.append(srd.signature);
      }
      else {
        ArrayReflectorDescriptor ard = new ArrayReflectorDescriptor (srd, asd);        
        nameBuffer.append(ard.signature);
      }  
   }

}
